/*
 *  Copyright (c) 2020 Rockchip Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __GENMESH_H__
#define __GENMESH_H__

#define INV_POLY_COEFF_NUM 21								/* ʽϵ, ߴ(INV_POLY_COEFF_NUM-1) */

enum RKALGO_LDCH_VERSION_E
{
    RKALGO_LDCH_VERSION_0 = 0,  /* old version: for rv1109rv1126rk356xrk3588 */
    RKALGO_LDCH_VERSION_1 = 1,  /* new version: for 1106rk3562 */
    RKALGO_LDCH_VERSION_BUTT
};

 /*  */
struct CameraCoeff
{
	double cx, cy;											/* ͷĹ */
	double a0, a2, a3, a4;									/* ͷĻϵ */
	double c, d, e;											/* ڲ[c d;e 1] */
	double sf;												/* sfӽ, sfԽӽԽ */

	/* level = 0ʱrho-tanThetaʽ */
	int invPolyTanNum0;										/* Ϻϵ */
	double invPolyTanCoeff0[INV_POLY_COEFF_NUM];		/* ʽϵ, ߴ(INV_POLY_COEFF_NUM-1) */
	/* level = 0ʱrho-cotThetaʽ */
	int invPolyCotNum0;										/* Ϻϵ */
	double invPolyCotCoeff0[INV_POLY_COEFF_NUM];		/* ʽϵ, ߴ(INV_POLY_COEFF_NUM-1) */

	/* level = 255ʱrho-tanThetaʽ */
	int invPolyTanNum255;									/* Ϻϵ */
	double invPolyTanCoeff255[INV_POLY_COEFF_NUM];		/* ʽϵ, ߴ(INV_POLY_COEFF_NUM-1) */
	/* level = 255ʱrho-cotThetaʽ */
	int invPolyCotNum255;									/* Ϻϵ */
	double invPolyCotCoeff255[INV_POLY_COEFF_NUM];		/* ʽϵ, ߴ(INV_POLY_COEFF_NUM-1) */
};

/* FECӳصĲ */
struct FecParams
{
	int correctX;										/* ˮƽxУ: 1У, 0У */
	int correctY;										/* ֱyУ: 1У, 0У */
	int saveMaxFovX;									/* ˮƽxFOV: 1, 0 */
	int isFecOld;										/* ǷɰFEC: 1ǣ0 */
	int saveMesh4bin;									/* Ƿ񱣴meshxi,xf,yi,yf4binļ1, 0 */
	char mesh4binPath[256];								/* meshxi,xf,yi,yf4binļ· */
	int srcW, srcH, dstW, dstH;							/* ͼķֱ */
	int srcW_ex, srcH_ex, dstW_ex, dstH_ex;				/* չֱ */
	double cropStepW[2000], cropStepH[2000];
	double cropStartW[2000], cropStartH[2000];
	int meshSizeW, meshSizeH;
	double meshStepW, meshStepH;
	int meshSize1bin;
	int meshSize4bin;
	unsigned short	SpbNum;
	unsigned long	MeshPointNumW;
	unsigned short	SpbMeshPNumH;
	unsigned short	LastSpbMeshPNumH;

	unsigned short	*pMeshXY;

};

/* LDCHӳصĲ */
struct LdchParams
{
	int saveMaxFovX;									/* ˮƽxFOV: 1, 0 */
    RKALGO_LDCH_VERSION_E enLdchVersion;			    /* LDCH汾 */
	int saveMeshX;										/* Ƿ񱣴MeshX.binļ: 1, 0 */
	char meshPath[256];									/* MeshX.binļ· */
	int srcW, srcH, dstW, dstH;							/* ͼķֱ */
	int meshSizeW, meshSizeH;
	double meshStepW, meshStepH;
	int mapxFixBit;										/* 㻯λ */
	int meshSize;
	int maxLevel;
	double *mapx;
	double *mapy;
};


/* =============================================================================================================================================================================== */

/* FEC: ʼ, ͼֱ, FECӳز, Ҫbuffer */
void genFecMeshInit(int srcW, int srcH, int dstW, int dstH, FecParams &fecParams, CameraCoeff &camCoeff);

/* FEC: ʼ */
void genFecMeshDeInit(FecParams &fecParams);

/* FEC: ԤȼĲ: δУСlevel=0,level=255Ķʽ */
void genFecPreCalcPart(FecParams &fecParams, CameraCoeff &camCoeff);

/* FEC: 4mesh ڴ */
void mallocFecMesh(int meshSize, unsigned short **pMeshXI, unsigned char **pMeshXF, unsigned short **pMeshYI, unsigned char **pMeshYF);

/* FEC: 4mesh ڴͷ */
void freeFecMesh(unsigned short *pMeshXI, unsigned char *pMeshXF, unsigned short *pMeshYI, unsigned char *pMeshYF);

/*
: ɲͬУ̶ȵmeshӳ, ISPFECģ
	:
	1FECӳز, Ҫbuffer: FecParams &fecParams
	2궨: CameraCoeff &camCoeff
	3ҪУĳ̶: level(0-255: 0ʾУ̶Ϊ0%, 255ʾУ̶Ϊ100%)
	:
	1bool Ƿɹ
	2pMeshXI, pMeshXF, pMeshYI, pMeshYF
*/
bool genFECMeshNLevel(FecParams &fecParams, CameraCoeff &camCoeff, int level, unsigned short *pMeshXI, unsigned char *pMeshXF, unsigned short *pMeshYI, unsigned char *pMeshYF);


/* =============================================================================================================================================================================== */

/* LDCH: ʼ, ͼֱ, LDCHӳز, Ҫbuffer */
void genLdchMeshInit(int srcW, int srcH, int dstW, int dstH, LdchParams &ldchParams, CameraCoeff &camCoeff);

/* LDCH: ʼ */
void genLdchMeshDeInit(LdchParams &ldchParams);

/* LDCH: ԤȼĲ: δУСlevel=0,level=255Ķʽ */
void genLdchPreCalcPart(LdchParams &ldchParams, CameraCoeff &camCoeff);

/* LDCH: LDCHܹУ̶ */
void calcLdchMaxLevel(LdchParams &ldchParams, CameraCoeff &camCoeff);

/*
: ɲͬУ̶ȵmeshӳ, ISPLDCHģ

	:
	1LDCHӳز, Ҫbuffer: LdchParams &ldchParams
	2궨: CameraCoeff &camCoeff
	3ҪУĳ̶: level(0-255: 0ʾУ̶Ϊ0%, 255ʾУ̶Ϊ100%)
	:
	1bool Ƿɹ
	2pMeshX
*/
bool genLDCMeshNLevel(LdchParams &ldchParams, CameraCoeff &camCoeff, int level, unsigned short *pMeshX);

/* =============================================================================================================================================================================== */

/* ͼROIز */
struct RoiParams
{
	int startW;		/* ROIʼ */
	int startH;
	int roiW;		/* ROIĿ */
	int roiH;
};

/* ͼROIFEC meshвü, õĳߴ */
bool cropFecMesh(FecParams &fecParams, RoiParams &roiParams, int level, unsigned short *pCropMeshXI, unsigned char *pCropMeshXF, unsigned short *pCropMeshYI, unsigned char *pCropMeshYF);

/* ͼROILDCH meshвü, õĳߴ */
bool cropLdchMesh(LdchParams &ldchParams, RoiParams &roiParams, int level, unsigned short *pMeshX, unsigned short *pRoiMeshX);

/* FEC: ʼ, 8k ---> 24k */
void genFecMeshInit8kTo4k(int srcW, int srcH, int dstW, int dstH, int margin,
	CameraCoeff &camCoeff, CameraCoeff &camCoeff_left, CameraCoeff &camCoeff_right,
	FecParams &fecParams, FecParams &fecParams_left, FecParams &fecParams_right);

/* LDCH: 8k ---> 24k */
void genLdchMeshInit8kTo4k(int srcW, int srcH, int dstW, int dstH, int margin,
	CameraCoeff &camCoeff, CameraCoeff &camCoeff_left, CameraCoeff &camCoeff_right,
	LdchParams &ldchParams, LdchParams &ldchParams_left, LdchParams &ldchParams_right);

#endif // !__GENMESH_H__
